/*
 * vim:ts=4:sw=4:expandtab
 *
 * © 2010 Michael Stapelberg <michael@stapelberg.de>
 * RELEASED INTO PUBLIC DOMAIN, WITHOUT ANY WARRANTY
 *
 * IPv6-socket, welcher auch IPv4-Verbindungen bedient
 *
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <err.h>

int main() {
    int ret;
    int fd = -1;

    struct addrinfo *res;
    struct addrinfo hints;
    memset(&hints, 0, sizeof(struct addrinfo));
    hints.ai_family = PF_INET6;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags = AI_PASSIVE;
    if ((ret = getaddrinfo(NULL, "8080", &hints, &res)) != 0)
        errx(EXIT_FAILURE, "getaddrinfo(): %s\n", gai_strerror(ret));
    struct addrinfo *walk;
    for (walk = res; walk != NULL; walk = walk->ai_next) {
        char buf[INET6_ADDRSTRLEN+1];
        memset(buf, 0, INET6_ADDRSTRLEN+1);
        if ((ret = getnameinfo((struct sockaddr*)walk->ai_addr, walk->ai_addrlen,
                               buf, sizeof(buf), NULL, 0, NI_NUMERICHOST)) != 0)
            errx(EXIT_FAILURE, "getnameinfo(): %s\n", gai_strerror(ret));

        printf("Listening on %s, port 8080\n", buf);
        fd = socket(walk->ai_family, walk->ai_socktype, walk->ai_protocol);
        if (fd == -1) {
            warn("socket()");
            continue;
        }
        /* Die Option IPV6_V6ONLY gibt an, ob IPv6-Sockets tatsächlich
         * nur IPv6 nutzen sollen oder auch IPv4-Verbindung akzeptieren.
         * Der Standardwert für diese Option kommt unter Linux aus
         * /proc/sys/net/ipv6/bindv6only
         * Die Option IPV6_V6ONLY ist in RFC 3542 spezifiziert. */
        int off = 0;
        if (setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, &off, sizeof(off)) == -1) {
            warn("setsockopt(IPV6_V6ONLY)");
            fd = -1;
            continue;
        }
        if (bind(fd, walk->ai_addr, walk->ai_addrlen) != 0) {
            warn("bind()");
            fd = -1;
            continue;
        }
        break;
    }
    if (fd == -1)
        errx(EXIT_FAILURE, "Could not bind socket");

    if (listen(fd, 5) == -1)
        err(EXIT_FAILURE, "listen()");

    while (1) {
        struct sockaddr_storage peer;
        socklen_t peers = sizeof(struct sockaddr_storage);
        int cfd = accept(fd, (struct sockaddr*)&peer, &peers);

        char buf[INET6_ADDRSTRLEN+1];
        struct sockaddr_storage addr;
        int ret;

        socklen_t addrlen = sizeof(addr);
        if (getpeername(cfd, (struct sockaddr*)&addr, &addrlen) == -1)
            err(EXIT_FAILURE, "getsockname()");

        memset(buf, 0, INET6_ADDRSTRLEN+1);
        if ((ret = getnameinfo((struct sockaddr*)&addr, addrlen,
                               buf, sizeof(buf),
                               NULL, 0,
                               NI_NUMERICHOST)) != 0)
            errx(1, "getnameinfo(): %s\n", gai_strerror(ret));

        printf("New connection from %s\n", buf);
    }
}
