/*
 * vim:ts=4:sw=4:expandtab
 *
 * © 2010 Michael Stapelberg <michael@stapelberg.de>
 * RELEASED INTO PUBLIC DOMAIN, WITHOUT ANY WARRANTY
 *
 * Simpler Dual-Stack-Server
 *
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <err.h>

/* Natürlich sollte man hier über eine geeignetere Datenstruktur nachdenken
 * oder gleich eventbasiert programmieren (z.B. mit libev) */
int sockfds[10];

int main() {
    int ret, fd;
    int n = 0;

    struct addrinfo *res;
    struct addrinfo hints;
    memset(&hints, 0, sizeof(struct addrinfo));
    hints.ai_family = PF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags = AI_PASSIVE;
    if ((ret = getaddrinfo(NULL, "9090", &hints, &res)) != 0)
        errx(EXIT_FAILURE, "getaddrinfo(): %s\n", gai_strerror(ret));
    struct addrinfo *walk;
    for (walk = res; walk != NULL; walk = walk->ai_next) {
        char buf[INET6_ADDRSTRLEN+1];
        memset(buf, 0, INET6_ADDRSTRLEN+1);
        if ((ret = getnameinfo((struct sockaddr*)walk->ai_addr, walk->ai_addrlen,
                               buf, sizeof(buf), NULL, 0, NI_NUMERICHOST)) != 0)
            errx(EXIT_FAILURE, "getnameinfo(): %s\n", gai_strerror(ret));

        fd = socket(walk->ai_family, walk->ai_socktype, walk->ai_protocol);
        if (fd == -1) {
            warn("socket()");
            continue;
        }

        printf("Listening on %s, port 9090 (fd %d)\n", buf, fd);
        if (walk->ai_family == AF_INET6) {
            int off = 1;
            if (setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, &off, sizeof(off)) == -1) {
                warn("setsockopt(IPV6_V6ONLY)");
                continue;
            }
        }
        if (bind(fd, walk->ai_addr, walk->ai_addrlen) != 0) {
            warn("bind()");
            continue;
        }

        if (listen(fd, 5) == -1) {
            warn("listen()");
            continue;
        }

        sockfds[n++] = fd;
    }

    fd_set fds;
    int highest;
    while (1) {
        highest = 0;
        FD_ZERO(&fds);
        /* Auf Aktivität auf einem der Server-Sockets warten */
        for (int i = 0; i < n; i++) {
            FD_SET(sockfds[i], &fds);
            highest = (sockfds[i] > highest ? sockfds[i] : highest);
        }
        ret = select(highest+1, &fds, NULL, NULL, NULL);
        if (ret == -1)
            err(EXIT_FAILURE, "select()");

        for (int i = 0; i < n; i++) {
            if (!FD_ISSET(sockfds[i], &fds))
                continue;

            struct sockaddr_storage peer;
            socklen_t peers = sizeof(struct sockaddr_storage);
            int cfd = accept(sockfds[i], (struct sockaddr*)&peer, &peers);

            char buf[INET6_ADDRSTRLEN+1];
            struct sockaddr_storage addr;
            int ret;

            socklen_t addrlen = sizeof(addr);
            if (getpeername(cfd, (struct sockaddr*)&addr, &addrlen) == -1)
                err(EXIT_FAILURE, "getsockname()");

            memset(buf, 0, INET6_ADDRSTRLEN+1);
            if ((ret = getnameinfo((struct sockaddr*)&addr, addrlen,
                                   buf, sizeof(buf),
                                   NULL, 0,
                                   NI_NUMERICHOST)) != 0)
                errx(EXIT_FAILURE, "getnameinfo(): %s\n", gai_strerror(ret));

            printf("New connection on fd %d from %s\n", sockfds[i], buf);
        }
    }
}
