/*
 * vim:ts=4:sw=4:expandtab
 *
 * © 2010 Michael Stapelberg <michael@stapelberg.de>
 * RELEASED INTO PUBLIC DOMAIN, WITHOUT ANY WARRANTY
 *
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <err.h>

static int open_connection(struct addrinfo *res) {
    int fd = -1;
    struct addrinfo *walk;
    for (walk = res; walk != NULL; walk = walk->ai_next) {
        fd = socket(walk->ai_family, walk->ai_socktype,
                    walk->ai_protocol);
        if (fd < 0) {
            warn("socket()");
            continue;
        }
        if (connect(fd, walk->ai_addr, walk->ai_addrlen) < 0) {
            /* Fehler melden, aber nicht abbrechen -- möglicherweise
             * folgen noch andere Adressen, bei denen die Verbindung
             * klappt. */
            warn("could not connect()");
            close(fd);
            fd = -1;
            continue;
        }
        return fd;
    }
    return -1;
}

int main() {
    int ret;

    struct addrinfo *res;
    struct addrinfo hints;
    memset(&hints, 0, sizeof(struct addrinfo));
    hints.ai_family = PF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    if ((ret = getaddrinfo("i3.zekjur.net", "http", &hints, &res)) != 0)
        errx(EXIT_FAILURE, "getaddrinfo(i3.zekjur.net) = %s\n", gai_strerror(ret));

    int sockfd;
    sockfd = open_connection(res);
    freeaddrinfo(res);
    if (sockfd == -1)
        errx(EXIT_FAILURE, "Could not connect\n");
    printf("Successfully connected to i3.zekjur.net:http\n");
}
