/*
 * vim:ts=4:sw=4:expandtab
 *
 * © 2010 Michael Stapelberg <michael@stapelberg.de>
 * RELEASED INTO PUBLIC DOMAIN, WITHOUT ANY WARRANTY
 *
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <err.h>

static int parse_address(const char *address, const char *defport,
                         struct addrinfo **addr) {
    char *host, *copy;
    char *port;
    if ((host = copy = strdup(address)) == NULL)
        err(1, "strdup()");
    printf("Parsing %s:\n", address);

    if (*host == '[') {
        host++;
        if ((port = strchr(host, ']')) != NULL) {
            *port = '\0';
            port++;
            if (*port == ':')
                port++;
            else port = (char*)defport;
        }
    } else {
        if ((port = strchr(host, ':')) != NULL) {
            *port = '\0';
            port++;
        } else port = (char*)defport;
    }

    printf("host = %s, port = %s\n", host, port);

    int ret;
    if ((ret = getaddrinfo(host, port, NULL, addr)) != 0)
        errx(EXIT_FAILURE, "getaddrinfo(): %s\n", gai_strerror(ret));

    struct addrinfo *res = *addr;

    char buf[INET6_ADDRSTRLEN+1];
    char portbuf[INET6_ADDRSTRLEN+1];

    memset(buf, 0, INET6_ADDRSTRLEN+1);
    memset(portbuf, 0, INET6_ADDRSTRLEN+1);
    if ((ret = getnameinfo((struct sockaddr*)res->ai_addr, res->ai_addrlen,
                           buf, sizeof(buf),
                           portbuf, sizeof(portbuf),
                           NI_NUMERICHOST | NI_NUMERICSERV)) != 0)
        errx(1, "getnameinfo(): %s\n", gai_strerror(ret));

    printf("Resolved to: %s on port %s\n", buf, portbuf);
    printf("\n");

    return 0;
}

int main() {
    struct addrinfo *addr;
    parse_address("streamsaver.sur5r.net", "443", &addr);
    parse_address("streamsaver.sur5r.net:80", "443", &addr);
    parse_address("192.168.1.24", "443", &addr);
    parse_address("192.168.1.24:80", "443", &addr);
    parse_address("[2001:470:9e42:0:250:baff:fe21:c27f]", "443", &addr);
    parse_address("[2a02:2e0:3fe:100::6]:90", "443", &addr);
    parse_address("[2a02:2e0:3fe:100::6]:http", "443", &addr);
    parse_address("[fe80::250:baff:fe21:c27f%eth0]:120", "443", &addr);
    parse_address("[::]:120", "443", &addr);
    parse_address("[::ffff:192.168.1.24]:120", "443", &addr);
}
